local scene = lt.Layer()

local samples = lt.LoadSamples({"blue_explosion", "blue_turn", "blue_whoosh"})
local models = lt.LoadModels({"thing"})
local images = lt.LoadImages({"vehicle"})

local player_node = images.vehicle:Scale(1, 1, 1):Translate(0, -45, 0):Rotate(0)

scene:Insert(player_node)

local turn_track = lt.Track()
turn_track:Queue(samples.blue_turn)
turn_track.loop = true
turn_track.playing = false

scene:Insert(turn_track)

local explosion = lt.Mesh()
do
    explosion.draw_mode = "lines"
    local num_rays = 40
    local verts = {}
    for i = 1, num_rays do
        local angle = ((i-1) / num_rays) * 360
        local x = cos(angle)
        local y = sin(angle)
        table.append(verts, {0, 0, x, y})
    end
    explosion:SetXYs(verts)
    explosion = explosion:Scale(0)
end

local angle = 0
local exploding = false
local speed_scaling = 1

local
function add_thing()
    local thing_speed = 40 + 20 * math.random()
    local a = math.random() * 360
    local z = -100
    local thing_node = models.thing:Scale(6, 6, 1):Translate(0, -50, z)
        :Rotate(a)
    thing_node:Action(function(dt)
        if exploding then return end
        z = z + thing_speed * dt * speed_scaling
        thing_node.z = z
        if z > 10 then
            scene:Remove(thing_node)
            add_thing()
            samples.blue_whoosh:Play()
            return true
        end
        if z > -2 and z < 2 then
            if math.abs((a%360) - (angle%360)) < 10 then
                samples.blue_explosion:Play()
                scene:Remove(thing_node)
                add_thing()
                player_node.child.child.child = explosion
                explosion.scale = 0
                exploding = true
                explosion:Tween{scale = 50, time = 0.5, action = function()
                    player_node.child.child.child = images.vehicle
                    exploding = false
                    speed_scaling = 1
                end}
            end
        end
    end)
    scene:Insert(thing_node)
end
for i = 1, 20 do
    add_thing()
end

local pscene = scene:Perspective(1, 10, 100)
    :DepthTest(true)
    --[[
    :Light{
        x = 0,
        y = 0,
        z = 0,
        l = 0.0000,
        q = 0.0000,
        ambient_red = 0.5,
        ambient_green = 0.5,
        ambient_blue = 0.5,
        diffuse_red = 1,
        diffuse_green = 1,
        diffuse_blue = 1,
        fixed = false,
    }
    :Lighting(true)
    ]]

local z = 0
local xdir = "none"
local zdir = "none"
local xspeed = 130
local zspeed = 40

pscene:Action(function(dt)
    if exploding then return end
    if xdir == "left" then
        angle = angle - xspeed * dt
    elseif xdir == "right" then
        angle = angle + xspeed * dt
    end
    player_node.angle = angle
    if zdir == "up" then
        z = z - zspeed * dt
    elseif zdir == "down" then
        z = z + zspeed * dt
    end
    if z < -50 then
        z = -50
    elseif z > 0 then
        z = 0
    end
    --player_node.z = z
    speed_scaling = speed_scaling + dt * 0.1
    if xdir ~= "none" and not turn_track.playing then
        turn_track:Play()
        turn_track.pitch = speed_scaling
        turn_track.playing = true
    elseif xdir == "none" and turn_track.playing then
        turn_track:Stop()
        turn_track.playing = false
    end
end)

function pscene:key_down(event)
    if event.key == "left" then
        xdir = "left"
    elseif event.key == "right" then
        xdir = "right"
    elseif event.key == "up" then
        zdir = "up"
    elseif event.key == "down" then
        zdir = "down"
    end
end

function pscene:key_up(event)
    if event.key == "left" and xdir == "left" then
        xdir = "none"
    elseif event.key == "right" and xdir == "right" then
        xdir = "none"
    elseif event.key == "up" and zdir == "up" then
        zdir = "none"
    elseif event.key == "down" and zdir == "down" then
        zdir = "none"
    end
end

return pscene
