local scene = lt.Layer()

local samples = lt.LoadSamples({"green_thrust", "green_fire", "meteor_explode", "green_explode"})
local images = lt.LoadImages({"glow", "ship", "afterburner"})


local vx, vy = 0, 0
local ship_x, ship_y = 0, 0
local angle = 0
local thrusting = false
local firing = false
local turn_dir = "none"
local turn_speed = 170
local thrust_accel = 70
local camera_buffer = 20
local camera_speed = 30
local camera_x, camera_y = ship_x, ship_y
local num_stars = 100
local meteors = {}
local max_meteors = 4
local exploding = false

local ship_layer = lt.Layer()

local ship_node = lt.Layer(images.ship):Scale(0.5):Rotate(0):Translate(ship_x, ship_y)

ship_layer:Insert(ship_node)

local camera = ship_layer:Translate(0, 0)
scene:Insert(camera)

local thrust_track = lt.Track()
thrust_track:Queue(samples.green_thrust)
thrust_track.loop = true

scene:Insert(thrust_track)

local stars_mesh = lt.Mesh()
stars_mesh.draw_mode = "points"
local star_verts = {}
for i = 1, num_stars * 2, 2 do
    star_verts[i] = math.random() * width - width/2
    star_verts[i+1] = math.random() * height - height/2
end
stars_mesh:SetXYs(star_verts)
stars_mesh:Action(function(dt)
    for i = 1, num_stars * 2, 2 do
        local sx = star_verts[i]
        local sy = star_verts[i+1]
        if sx > camera_x + width/2 then
            sx = sx - width
        elseif sx < camera_x - width/2 then
            sx = sx + width
        end
        if sy > camera_y + height/2 then
            sy = sy - width
        elseif sy < camera_y - height/2 then
            sy = sy + width
        end
        star_verts[i] = sx
        star_verts[i+1] = sy
    end
    stars_mesh:SetXYs(star_verts)
end)

ship_layer:Insert(stars_mesh)

scene:Action(function(dt)
    if exploding then return end
    if turn_dir == "left" then
        angle = angle + dt * turn_speed
    elseif turn_dir == "right" then
        angle = angle - dt * turn_speed
    end
    ship_node.angle = angle
    if thrusting then
        local ax = cos(angle) * thrust_accel
        local ay = sin(angle) * thrust_accel
        vx = vx + ax * dt
        vy = vy + ay * dt
    end
    if thrusting and math.random(2) == 1 then
        ship_node:Insert(images.afterburner)
    else
        ship_node:Remove(images.afterburner)
    end
    ship_x = ship_x + vx * dt
    ship_y = ship_y + vy * dt
    ship_node.x = ship_x
    ship_node.y = ship_y

    camera_x = ship_x + math.min(40, math.max(-40, vx*0.1))
    camera_y = ship_y + math.min(40, math.max(-40, vy*0.1))
    camera.x = -camera_x
    camera.y = -camera_y

    local i = 1
    while i <= #meteors do
        local meteor = meteors[i]
        local d = math.sqrt((ship_x-meteor.x)^2 + (ship_y-meteor.y)^2)
        if d < meteor.radius then
            for i = 1, 5 do
                local line = lt.Mesh()
                line.draw_mode = "lines"
                line:SetXYs{-4, 0, 4, 0}
                local angle = math.random() * 360
                local x, y = math.random() * 10 - 5 + ship_x, math.random() * 10 - 5 + ship_y
                local tx, ty, ta = math.random() * 40 - 20 + ship_x, math.random() * 40 - 20 + ship_y, math.random() * 360
                line = line:Rotate(angle):Translate(x, y)
                line:Tween{
                    x = tx,
                    y = ty,
                    angle = ta,
                    time = 1,
                    action = function()
                        ship_layer:Remove(line)
                        exploding = false
                        ship_node:Insert(images.ship)
                    end
                }
                ship_layer:Insert(line)
            end
            ship_node:Remove(images.ship)
            exploding = true
            for _, meteor in ipairs(meteors) do
                ship_layer:Remove(meteor.node)
            end
            meteors = {}
            vx = 0
            vy = 0
            ship_x = 0
            ship_y = 0
            samples.green_explode:Play()
        end
        if d > 100 then
            ship_layer:Remove(meteor.node)
            table.remove(meteors, i)
        else
            i = i + 1
        end
    end
end)

local explode_meteor

local bullet_speed = 50
scene:Action(function(dt)
    if firing and not exploding then
        local bx, by = ship_x, ship_y
        local speed_x = bullet_speed * cos(angle) + vx
        local speed_y = bullet_speed * sin(angle) + vy
        local bullet = images.glow:Scale(0.1):Translate(bx, by)
        ship_layer:Insert(bullet)
        local t = 0
        bullet:Action(function(dt)
            bx = bx + speed_x * dt
            by = by + speed_y * dt
            bullet.x = bx
            bullet.y = by
            t = t + dt
            if t > 1 then
                ship_layer:Remove(bullet)
                return true
            end
            local i = 1
            while i <= #meteors do
                local meteor = meteors[i]
                if math.sqrt((bx-meteor.x)^2+(by-meteor.y)^2) < meteor.radius then
                    table.remove(meteors, i)
                    ship_layer:Remove(meteor.node)
                    explode_meteor(meteor)
                    ship_layer:Remove(bullet)
                    return true
                end
                i = i + 1
            end
        end)
        samples.green_fire:Play()
    end
    return 0.2
end)

local
function make_meteor(x, y, radius)
    local mesh = lt.Mesh()
    mesh.draw_mode = "line_loop"
    local angle = 0
    local verts = {}
    while angle < 360 do
        table.append(verts, {cos(angle) * radius, sin(angle) * radius})
        angle = angle + math.random(40) + 30
    end
    mesh:SetXYs(verts)
    local angle = 0
    local va = math.random() * 60 - 30
    local speed = (math.random() * 30 + 10) * (3/radius)
    local node = mesh:Rotate(angle):Translate(x, y)
    ship_layer:Insert(node)
    local meteor = {
        node = node,
        x = x,
        y = y,
        radius = radius
    }
    node:Action(function(dt)
        angle = angle + va * dt
        local dx = ship_x - x
        local dy = ship_y - y
        local d = (dx^2+dy^2)^0.5
        x = x + dt * (dx/d) * speed
        y = y + dt * (dy/d) * speed
        node.angle = angle
        node.x = x
        node.y = y
        meteor.x = x
        meteor.y = y
    end)
    table.insert(meteors, meteor)
end

function explode_meteor(meteor)
    samples.meteor_explode:Play()
    if meteor.radius < 3 then
        return
    end
    for i = 1, 3 do
        make_meteor(meteor.x + math.random()*10-5, meteor.y + math.random()*10-5, meteor.radius/2)
    end
end

scene:Action(function(dt)
    if exploding then return end
    while #meteors < max_meteors do
        local radius = math.random() * 15 + 2
        local angle = math.random() * 360
        local d = 60 + radius
        local x = cos(angle) * d + ship_x
        local y = sin(angle) * d + ship_y
        make_meteor(x, y, radius)
    end
end)

function scene:key_down(event)
    if event.key == "up" then
        thrusting = true
        thrust_track:Play()
    elseif event.key == "down" then
        firing = true
    elseif event.key == "left" then
        turn_dir = "left"
    elseif event.key == "right" then
        turn_dir = "right"
    end
end

function scene:key_up(event)
    if event.key == "up" then
        thrusting = false
        thrust_track:Pause()
    elseif event.key == "down" then
        firing = false
    elseif event.key == "left" and turn_dir == "left" then
        turn_dir = "none"
    elseif event.key == "right" and turn_dir == "right" then
        turn_dir = "none"
    end
end

return scene
