local scene = lt.Layer():Tint(1, 1, 1, 0.25)

local samples = lt.LoadSamples({"punch", "grunt1", "grunt2", "land"})
local images = lt.LoadImages({"fighter_stand", "fighter_punch", "punch"})

local x, y = -40, -20
local ox, oy = 40, -20
local player_node = images.fighter_stand:Translate(2, 0):Scale(1, 1):Translate(x, y)
local opponent_node = images.fighter_stand:Translate(2, 3):Scale(-1, 1.3):Translate(ox, oy)
local ppunching = false
local opunching = false
local pfalling = false
local ofalling = false

scene:Insert(player_node)
scene:Insert(opponent_node)

local dir = "none"
local speed = 60
local vy = 0

local odir = "none"
local ovy = 0

player_node:Action(function(dt)
    if pfalling then return end
    if dir == "left" then
        x = x - dt * speed
    elseif dir == "right" then
        x = x + dt * speed
    end
    if x < -40 then
        x = -40
    elseif x > 40 then
        x = 40
    end
    player_node.x = x

    y = y + vy * dt
    if y < -20 then
        y = -20
        vy = 0
    elseif y > 20 then
        vy = -80
    end
    player_node.y = y

    if ox < x then
        player_node.scale_x = -1
        opponent_node.scale_x = 1
    else
        player_node.scale_x = 1
        opponent_node.scale_x = -1
    end

    if opunching and not pfalling then
        if math.abs(ox-x) < 25 then
            local punch = images.punch:Scale(0.5):Rotate(0):Translate(ox+20*opponent_node.scale_x, oy+10)
            scene:Insert(punch)
            punch:Tween{scale = 1, angle = 90, time = 0.5, action = function()
                scene:Remove(punch)
            end}
            player_node.child.child.child = images.fighter_stand:Rotate(90)
            pfalling = true
            local tx1 = x - 20 * player_node.scale_x
            player_node:Tween{y = 20, x = tx1, time = 0.5, action = function()
                samples.land:Play()
                player_node:Tween{y = -27, time = 1.0, easing = "bounce", action = function()
                    pfalling = false
                    player_node.child.child.child = images.fighter_stand
                    x = tx1
                end}
            end}
            samples.punch:Play()
        end
    elseif ppunching and not ofalling then
        if math.abs(ox-x) < 20 then
            local punch = images.punch:Scale(0.5):Rotate(0):Translate(x+20*player_node.scale_x, y+5)
            scene:Insert(punch)
            punch:Tween{scale = 1, angle = 90, time = 0.5, action = function()
                scene:Remove(punch)
            end}
            opponent_node.child.child.child = images.fighter_stand:Rotate(90)
            ofalling = true
            local tx1 = ox - 20 * opponent_node.scale_x
            opponent_node:Tween{y = 20, x = tx1, time = 0.5, action = function()
                samples.land:Play(0.9)
                opponent_node:Tween{y = -27, time = 1.0, easing = "bounce", action = function()
                    ofalling = false
                    opponent_node.child.child.child = images.fighter_stand
                    ox = tx1
                end}
            end}
            samples.punch:Play()
        end
    end
end)

opponent_node:Action(function(dt)
    if ofalling then return end
    if odir == "left" then
        ox = ox - dt * speed
    elseif odir == "right" then
        ox = ox + dt * speed
    end
    if ox < -40 then
        ox = -40
    elseif ox > 40 then
        ox = 40
    end
    opponent_node.x = ox

    oy = oy + ovy * dt
    if oy < -20 then
        oy = -20
        ovy = 0
    elseif oy > 20 then
        ovy = -80
    end
    opponent_node.y = oy
end)

opponent_node:Action(function()
    if ofalling then return end
    local r = math.random(4)
    if r == 1 then
        odir = "left"
    elseif r == 2 then
        odir = "right"
    elseif r == 3 and oy < -15 then
        samples.grunt2:Play(0.9)
        ovy = 100
    elseif r == 4 then
        opponent_node.child.child.child = images.fighter_punch
        opunching = true
        samples.grunt1:Play(0.9)
        local alt = false
        opponent_node:Action(function()
            if alt then
                opponent_node.child.child.child = images.fighter_stand
                opunching = false
                return true
            else
                alt = true
                return 0.5
            end
        end)
    end
    return math.random() * 0.5 + 0.3
end)

function scene:key_down(event)
    if pfalling then return end
    if event.key == "down" then
        player_node.child.child.child = images.fighter_punch
        ppunching = true
        samples.grunt1:Play()
    elseif event.key == "left" then
        dir = "left"
    elseif event.key == "right" then
        dir = "right"
    elseif event.key == "up" and y < -15 then
        samples.grunt2:Play()
        vy = 100
    end
end

function scene:key_up(event)
    if pfalling then return end
    if event.key == "down" then
        player_node.child.child.child = images.fighter_stand
        ppunching = false
    elseif event.key == "left" and dir == "left" then
        dir = "none"
    elseif event.key == "right" and dir == "right" then
        dir = "none"
    end
end

return scene
