lt.FixGlobals()

local play

local images = lt.LoadImages({"overlay", "reflection1", "title",
    {font="font", glyphs="0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ.:"}})

local samples = lt.LoadSamples({"music", "ocean", "gulls"})
local ocean_track = lt.Track()
ocean_track:Queue(samples.ocean)
ocean_track.loop = true
ocean_track.gain = 0.2
ocean_track:Play()
local gulls_track = lt.Track()
gulls_track:Queue(samples.gulls)
gulls_track.loop = true
gulls_track.gain = 0.5
gulls_track:Play()
local text = lt.Text("POCKET MONEY: R2.00", images.font, "left", "top")
    :Scale(0.35)
    :Tint(1, 1, 1, 0.6)
    :BlendMode("add")
    :Translate(-48, 48)
lt.root.child = lt.Layer(text, images.title:Scale(1/2),
    ocean_track, gulls_track)
lt.root.child:KeyDown(function(event)
    if event.key == "enter" then
        ocean_track:Stop()
        ocean_track = nil
        gulls_track:Stop()
        gulls_track = nil
        play()
    elseif event.key == "esc" then
        lt.Quit()
    end
end)

function play()
    local pocket_money = lt.Text("POCKET MONEY: R2.00", images.font, "left", "top")
        :Scale(0.35)
        :Tint(1, 1, 1, 0.6)
        :Translate(-48, 48)

    local gameover = false

    local scene = lt.Layer()
    lt.root.child = scene

    local music = lt.Track()
    music.loop = true
    music.gain = 0.05
    music:Queue(samples.music)

    local red_game = import("red_game")
    local green_game = import("green_game")
    local blue_game = import("blue_game")
    local magenta_game = import("magenta_game")

    local diming = 5
    local overlay = lt.Rect(lt.left, lt.bottom, lt.right, lt.top):BlendMode("subtract"):Tint(1, 1, 1, 0)

    local target = lt.RenderTarget(nil, 200, 200,
        lt.left, lt.bottom, lt.right, lt.top,
        lt.left, lt.bottom, lt.right, lt.top,
        "linear", "linear")

    local n = 50
    local mesh = target:Mesh():Grid(n, n)
    local xys = {}
    local col_width, row_height = 2, 2
    local y = -50
    local
    function distort(x)
        local a = 1 - math.abs(x / 50)
        a = 1 - math.pow(a, 1.4)
        if x < 0 then
            a = a * -1
        end
        return a * 50
    end
    for row = 0, n do
        local x = -50
        for col = 0, n do
            table.append(xys, {distort(x), distort(y)})
            x = x + col_width
        end
        y = y + row_height
    end
    mesh:SetXYs(xys)

    local me = images.reflection1:Translate(0, -35):Scale(1/3):Translate(0, 12):Tint(1, 1, 1):BlendMode("add")
    me:Action(function(dt)
        me.red = math.random()
        me.green = math.random()
        me.blue = math.random()
        --me.child.child.child.child = images["reflection"..math.random(2)]
        return math.random() + 0.1
    end)
    me:Tween{delay = 15, scale = 8, easing = "zoomin", time = 60, action = function()
        scene:Remove(music)
        scene:Insert(import "gameover")
        gameover = true
    end}
    music:Tween{delay = 10, gain = 1, time = 60}

    scene:Insert(mesh)
    scene:Insert(red_game:Hidden())
    scene:Insert(green_game:Hidden())
    scene:Insert(blue_game:Hidden())
    scene:Insert(magenta_game:Hidden())
    scene:Insert(images.overlay:Scale(1/4))
    scene:Insert(me)
    scene:Insert(music)
    music:Play()

    local all_games = lt.Layer(
        red_game        :ColorMask(true, false, false, true),
        green_game      :ColorMask(false, true, false, true),
        blue_game       :ColorMask(false, false, true, true),
        magenta_game    :ColorMask(true, false, true, true)
    ):BlendMode("add")

    scene:Action(function(dt)
        overlay.alpha = dt * diming
        overlay:Draw(target)
        all_games:Draw(target)
    end)

    scene:KeyDown(function(event)
        if event.key == "esc" then
            lt.Quit()
        end
        if gameover then return end
        red_game:key_down(event)
        green_game:key_down(event)
        blue_game:key_down(event)
        magenta_game:key_down(event)
    end)

    scene:KeyUp(function(event)
        if gameover then return end
        red_game:key_up(event)
        green_game:key_up(event)
        blue_game:key_up(event)
        magenta_game:key_up(event)
    end)

    scene:Insert(pocket_money)

    local t = 0
    local money = 2.2
    pocket_money:Action(function(dt)
        money = money - 0.2
        if money < 0 then money = 0 end
        pocket_money.child.child.child = 
            lt.Text(string.format("POCKET MONEY: R%0.2f", money), images.font, "left", "top")
        return 75/11
    end)
end
