local scene = lt.Layer()

local samples = lt.LoadSamples({"red_collect", "red_die"})
local player_node = lt.Rect(-1, -1, 1, 1):Translate(0, 0)

local dir = "up"
local trail = {}
local trail_len = 10
local pills = {}
local max_pills = 4

local speed = 30
local x, y = 0, 0

local tnode = lt.Rect(-1, -1, 1, 1):Scale(1):Tint(1, 1, 1, 1)

local exploding = false

scene:Action(function(dt)
    if exploding then return end
    if dir == "up" then
        y = y + speed * dt
    elseif dir == "down" then
        y = y - speed * dt
    elseif dir == "left" then
        x = x - speed * dt
    elseif dir == "right" then
        x = x + speed * dt
    end
    if x > lt.right then
        x = lt.left
    elseif x < lt.left then
        x = lt.right
    end
    if y > lt.top then
        y = lt.bottom
    elseif y < lt.bottom then
        y = lt.top
    end
    player_node.x = x
    player_node.y = y

    for i, pill in ipairs(pills) do
        if math.sqrt((x - pill.x)^2 + (y - pill.y)^2) < 2 then
            scene:Remove(pill.node)
            table.remove(pills, i)
            trail_len = trail_len + 10
            samples.red_collect:Play()
        end
    end

    for i = 1, #trail - 1 do
        local piece = trail[i]
        if math.sqrt((x-piece.x)^2 + (y-piece.y)^2) < 2 then
            exploding = true
            samples.red_die:Play()
            local overlay = lt.Rect(-50, -50, 50, 50):Tint(1, 1, 1, 0.3)
            scene:Insert(overlay)
            overlay:Tween{alpha = 0, time = 0.1, action = function()
                scene:Remove(overlay)
            end}
            tnode:Tween{scale = 4, time = 1, action = function()
                trail_len = 5
                exploding = false
                tnode.scale = 1
                tnode.alpha = 1
                while #trail > 0 do
                    local piece = trail[1]
                    scene:Remove(piece.node)
                    table.remove(trail, 1)
                end
            end}
        end
    end
end)

scene:Action(function(dt)
    while #pills < max_pills do
        local px, py = math.random() * 80 - 40, math.random() * 80 - 40
        local pill_node = lt.Rect(-1, -1, 1, 1):Rotate(45):Translate(px, py)
        scene:Insert(pill_node)
        local pill = {
            node = pill_node,
            x = px,
            y = py
        }
        table.insert(pills, pill)
    end
    return 2
end)

scene:Action(function(dt)
    local tx, ty = x, y
    local trail_node = tnode:Translate(tx, ty)
    scene:Insert(trail_node)
    local trail_piece = {
        node = trail_node,
        x = tx,
        y = ty
    }
    table.insert(trail, trail_piece)
    while #trail > trail_len do
        local piece = trail[1]
        scene:Remove(piece.node)
        table.remove(trail, 1)
    end
    return 0.1
end)

scene:Insert(player_node)

function scene:key_down(event)
    local key = event.key
    if key == "up" and dir ~= "down" or key == "down" and dir ~= "up" 
        or key == "left" and dir ~= "right" or key == "right" and dir ~= "left"
    then
        dir = key
    end
end

function scene:key_up(event)
end

return scene
